<?php

namespace Drupal\cafe_outlet_billing\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;

/**
 * {@inheritdoc}
 */
class CafeOutletBillingController extends ControllerBase {

  /**
   * Constructs a new CafeOutletBillingController.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
    );
  }

  /**
   * Function to generate the bill and create a PDF.
   *
   * @param string $cashier_id
   *   The cashier's user ID.
   * @param string $items
   *   JSON-encoded string of items (URL encoded).
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   PDF file as the response.
   */
  public function generateBill($cashier_id = '', $items = '[]') {
    $items = urldecode($items);
    $items = json_decode($items, TRUE);

    if (!is_array($items) || empty($items)) {
      return new Response('Invalid or no items provided', 400);
    }

    // Get site name and email from Drupal config.
    $site_name = $this->configFactory->get('system.site')->get('name');
    $site_email = $this->configFactory->get('system.site')->get('mail');
    $current_date = (new \DateTime())->format('d/m/Y');
    // Calculate the total for all items.
    $total = 0;
    foreach ($items as $item) {
      $quantity = (float) $item['quantity'];
      $price = (float) $item['price'];
      $total += $quantity * $price;
    }

    // Create a new PDF document.
    $pdf = new \TCPDF();
    $pdf->SetCreator(PDF_CREATOR);
    $pdf->SetTitle('Customer Bill');
    $pdf->AddPage();

    // Generate bill content.
    $html = <<<EOD
    <style>
      h1 { text-align: center; }
      .header { background-color: #F5F5F5; padding: 10px; }
      .item-row { border-bottom: 1px solid #E0E0E0; padding: 5px; }
      .totals-row { border-top: 2px solid red; padding: 5px; }
      .footer { background-color: #FFFAF0; padding: 10px; font-size: 12px; }
      .align-right { text-align: right; }
      .align-left { text-align: left; }
    </style>

    <h1>Invoice</h1>
  
    <div class="header">
      <div>
        <strong>Bill Generated by Cashier ID:</strong> {$cashier_id}
      </div>
      <div><strong> Invoice From:</strong> <br> {$site_name} <br> {$site_email} <br></div>
    </div>
    <table cellspacing="0" cellpadding="5" border="0" width="100%">
      <tr class="item-row">
        <th>Description</th>
        <th>QTY</th>
        <th>PRICE</th>
        <th>TOTAL</th>
      </tr>
EOD;

    foreach ($items as $item) {
      $item_name = $item['name'] ?: 'N/A';
      $quantity = (float) $item['quantity'];
      $price = (float) $item['price'];
      $item_total = $quantity * $price;

      $html .= <<<EOD
      <tr class="item-row">
        <td>{$item_name}</td>
        <td>{$quantity}</td>
        <td>{$price}</td>
        <td>{$item_total}</td>
      </tr>
EOD;
    }

    $html .= <<<EOD
      <tr class="totals-row">
        <td colspan="3" class="align-right"><strong>Total:</strong></td>
        <td>{$total}</td>
      </tr>
    </table>
    <div class="footer">
      <div class="align-right"><strong>DATE:</strong> $current_date</div>
    </div>
EOD;

    $pdf->writeHTML($html, TRUE, FALSE, TRUE, FALSE, '');

    // Output the PDF as a string.
    $pdf_output = $pdf->Output('bill.pdf', 'S');

    return new Response($pdf_output, 200, [
      'Content-Type' => 'application/pdf',
      'Content-Disposition' => 'inline; filename="bill.pdf"',
    ]);
  }

}
