<?php

namespace Drupal\cafe_outlet_billing\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * {@inheritdoc}
 */
class BillingForm extends FormBase {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   */
  public function __construct(AccountInterface $current_user) {
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cafe_outlet_billing_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $current_user = $this->currentUser;
    $cashier_id = $current_user->id();

    $form['cashier_id'] = [
      '#type' => 'textfield',
      '#default_value' => $cashier_id,
      '#attributes' => ['hidden' => 'hidden'],
    ];

    $items = $form_state->get('items');
    if ($items === NULL) {
      $items = [
        ['name' => 'Tea', 'quantity' => 1, 'price' => 10,'total'=>10],
        ['name' => 'Snack', 'quantity' => 1, 'price' => 20,'total'=>20],
      ];
      $form_state->set('items', $items);
    }

    $form['items_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'items-wrapper'],
    ];

    $form['items_wrapper']['items'] = [
      '#type' => 'table',
      '#header' => [$this->t('Item'), $this->t('Quantity'), $this->t('Price'), $this->t('Total')],
    ];

    // Generate form rows for each item.
    foreach ($items as $index => $item) {
      $form['items_wrapper']['items'][$index]['name'] = [
        '#type' => 'textfield',
        '#default_value' => $item['name'],
        '#attributes' => [
          'class' => ['item-name'],
          'maxlength' => 20,
        ],
      ];

      $form['items_wrapper']['items'][$index]['quantity'] = [
        '#type' => 'number',
        '#default_value' => $item['quantity'],
        '#min' => 0,
        '#attributes' => ['class' => ['item-quantity']],
      ];

      $form['items_wrapper']['items'][$index]['price'] = [
        '#type' => 'number',
        '#default_value' => $item['price'],
        '#min' => 0,
        '#attributes' => ['class' => ['item-price']],
      ];

      $form['items_wrapper']['items'][$index]['total'] = [
        '#markup' => '<span class="item-total">'.$item['total'].'</span>',
      ];

      // Remove button.
      $form['items_wrapper']['items'][$index]['remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('-'),
        '#submit' => ['::removeItem'],
        '#ajax' => [
          'callback' => '::ajaxCallback',
          'wrapper' => 'items-wrapper',
        ],
        '#limit_validation_errors' => [],
        '#item_index' => $index,
      ];
    }

    $form['items_wrapper']['add_item'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add Item'),
      '#submit' => ['::addItem'],
      '#attributes' => ['class' => ['item-add']],
      '#ajax' => [
        'callback' => '::ajaxCallback',
        'wrapper' => 'items-wrapper',
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Generate Bill'),
    ];

    $form['#attached']['library'][] = 'cafe_outlet_billing/billing_form';

    return $form;
  }

  /**
   * AJAX callback to refresh the item list when adding new items.
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state) {
    return $form['items_wrapper'];
  }

  /**
   * Adds a new item row to the form.
   */
  public function addItem(array &$form, FormStateInterface $form_state) {
    $items = $form_state->get('items');

    $items[] = ['name' => '', 'quantity' => 0, 'price' => 0];

    $form_state->set('items', $items);

    $form_state->setRebuild(TRUE);
  }

  /**
   * Removes an item row from the form.
   */
  public function removeItem(array &$form, FormStateInterface $form_state) {
    $item_index = $form_state->getTriggeringElement()['#item_index'];

    $items = $form_state->get('items');

    unset($items[$item_index]);

    // Reindex the array to maintain sequential indexing.
    $items = array_values($items);

    $form_state->set('items', $items);
    $form_state->setRebuild(TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $items = $form_state->getValue('items');

    if (is_array($items)) {
      // Serialize the items array to a JSON string.
      $items_json = json_encode($items);

      $form_state->setRedirect('cafe_outlet_billing.generate_bill', [
        'cashier_id' => $form_state->getValue('cashier_id'),
        'items' => $items_json,
      ]);
    }
    else {
    }
  }

}
